using dg1d
using GLMakie
using CairoMakie
using Random
GLMakie.activate!()


function sgibbs_map(x, ξ, d)
  s = x
  for (i, ξi) in enumerate(ξ)
    if (x > ξi)
      di = d[i]
      s += di + 1.0
    end
  end
  return s
end


const RNG = MersenneTwister()


function random_pos(eps)
  return eps * (rand(RNG) * 2.0 .- 1.0)
end



function my_bernstein_polynomial(x, n, N, a, b)
  return binomial(N, n) / (b - a)^N * (x - a)^n * (b - x)^(N-n)
end


function plot_mapped_bases(limit_w_bdrys = true)

  stepdata(x, x0) = [ (xi <= x0) ? -1.0 : 1.0 for xi in x ]

  # x0 = random_pos(1e-5)
  x0 = 0.5
  @show x0
  d = 1.0

  Nref = 100
  xref = collect(LinRange(-1.0, 1.0, Nref))
  fref = stepdata(xref, x0)
  xref_w_nans, fref_w_nans, bf_w_nans =
    [ dg1d.make_nan_separated_storage((Nref,1)) for _ = 1:3 ]
  xref_w_nans[1:end-1,:]  .= xref
  fref_w_nans[1:end-1,:]  .= fref

  sgibbs = (x) -> sgibbs_map(x, [x0], d)
  sxref = sgibbs.(xref)

  # Ns         = [1, 2, 3, 4, 5, 6, 7, 8, 9, 19, 29 ]
  Ns         = collect(1:2:30)
  meshes     = [ make_Mesh(N=N,K=1) for N in Ns ]
  mesh_sx    = [ sgibbs.(mesh.x) for mesh in meshes ]
  mesh_bx    = [ @. -1 + 2 * $(collect(0:N)) / N for N in Ns ]
  mesh_sbx   = [ sgibbs.(bx) for bx in mesh_bx ]
  Is         = [ dg1d.lagrange_interp_matrix(sxref, sx) for sx in mesh_sx ]
  # Also need to use smapped nodes for the projections and resummation matrices
  Ps         = [ dg1d.lagrange_interp_matrix(sbx, sx) for (sbx, sx) in zip(mesh_sbx, mesh_sx) ]
  Ss         = [ dg1d.vandermonde_matrix((x,j)->my_bernstein_polynomial(x,j,N,extrema(sx)...), sx)
                 for (N, sx) in zip(Ns, mesh_sx) ]


  markers = ['x', 'o', '■', '+', '◆']
  fig = Figure(resolution=(1600,1200))
  for (i,w) in enumerate([0,1/2,1])

    Label(fig[1,i], "w = $w", tellwidth=false)
    ax = Axis(fig[2,i])
    lines!(ax, xref_w_nans[:], fref_w_nans[:])
    for (idx, (N, mesh, I, P, S)) in enumerate(zip(Ns, meshes, Is, Ps, Ss))

      m = markers[Int64(floor(idx / 7)+1)]

      f = stepdata(mesh.x, x0)
      bf = copy(f)
      bfl, bfr = bf[1], bf[end]
      min_bf, max_bf = min(bfl, bfr), max(bfl, bfr)
      HRSC.reconstruct_cell!(bf, 1-w #= inconsistency with HRSC.reconstruct! =#, P, S,
                             m = limit_w_bdrys ? min_bf : missing,
                             M = limit_w_bdrys ? max_bf : missing)
      bf_w_nans[1:end-1,:] .= I * bf

      scatterlines!(ax, xref_w_nans[:], bf_w_nans[:], label="N = $N", markersize=10,
                    marker=m)
    end
    axislegend(ax, position=:lt)

  end

  return fig
end


# CairoMakie.activate!()
# save(joinpath(@__DIR__, "even_lagranges_fixed_left.pdf"), fig)
# GLMakie.activate!()
