export compute_rhs_weak_form!, 
       compute_rhs_strong_form!


"""
    compute_rhs_weak_form!(rhs, f, nf_lhs, nf_rhs, mesh)
    compute_rhs_weak_form!(rhs, f, s, nf_lhs, nf_rhs, mesh)
"""
function compute_rhs_weak_form!(rhs, f, nf_lhs, nf_rhs, mesh::AbstractMesh)
  @unpack invjac, element = mesh
  @unpack MDM, Ml_lhs, Ml_rhs, Npts = element
  @unpack K = mesh
  shape        = layout(mesh)
  mat_rhs      = reshape(view(rhs,:), shape)
  mat_f        = reshape(view(f,:), shape)
  # mat_MDM      = reshape(MDM, (Npts,Npts))
  # mul! currently allocates, although, (I think) it shouldn't ...
  # a = @allocated begin
  mul!(mat_rhs, MDM, mat_f)
  # end
  # display(a)
  @. mat_rhs  -= (nf_rhs' * Ml_rhs - nf_lhs' * Ml_lhs)
  mat_rhs    .*= invjac
  return
end
function compute_rhs_weak_form!(rhs, f, s, nf_lhs, nf_rhs, mesh::AbstractMesh)
  compute_rhs_weak_form!(rhs, f, nf_lhs, nf_rhs, mesh)
  rhs .+= s
  return
end


"""
    compute_rhs_strong_form!(rhs, f, nf_lhs, nf_rhs, mesh)
    compute_rhs_strong_form!(rhs, f, s, nf_lhs, nf_rhs, mesh)
"""
function compute_rhs_strong_form!(rhs, f, nf_lhs, nf_rhs, mesh::AbstractMesh)
  @unpack invjac, element = mesh
  @unpack Npts, D, Ml_lhs, Ml_rhs = element
  @unpack K = mesh
  shape        = layout(mesh)
  mat_rhs      = reshape(view(rhs,:), shape)
  mat_f        = reshape(view(f,:), shape)
  # compute rhs = - D * f inplace to rhs
  # benchmarking showed that splitting of the factor of -1 is faster
  # because a temporary is not created then
  mul!(mat_rhs, D, mat_f)
  mat_rhs     .*= -1
  @. mat_rhs  += (nf_rhs' * Ml_rhs - nf_lhs' * Ml_lhs)
  mat_rhs    .*= invjac
  return
end
function compute_rhs_strong_form!(rhs, f, s, nf_lhs, nf_rhs, mesh::AbstractMesh)
  compute_rhs_strong_form!(rhs, f, nf_lhs, nf_rhs, mesh)
  rhs .+= s
  return
end
