module GB


function fname_rule(n_pts)
  thisdir = dirname(@__FILE__)
  fname = "$thisdir/../misc/gegenbauer/rule_$n_pts"
  fname = normpath(fname)
  return fname
end # function



"""
Compute nodes and weights for Gegenbauer rule using external C++ program.
The C++ program dg1d/misc/gegenbauer/gegenbauer_rule should be build with the commands
```
  \$ cd dg1d/misc/gegenbauer
  \$ g++ gegenbauer_rule.cpp -o gegenbauer_rule
```

n_pts ... = order + 1
"""
function generate_nodes_weights(n_pts)

  # only interested in standard interval x ϵ [-1,1] with weight (1-t^2)^2
  α = 2.0
  A = -1.0
  B = 1.0

  thisdir = dirname(@__FILE__)
  exe = "$thisdir/../misc/gegenbauer/gegenbauer_rule"
  exe = normpath(exe)

  if ! isfile(exe)
    src = normpath("$thisdir/../misc/gegenbauer/gegenbauer_rule.cpp")
    run(`g++ $src -o $exe`)
  end

  fname = fname_rule(n_pts)

  pipe = pipeline(`$exe $n_pts $α $A $B $fname`, stdout=devnull, stderr=devnull)
  run(pipe)
end # function



"""
Read Gauss-Gegenbauer quadrature nodes and weights from file located at
evm/gegenbauer. If no file is found for requested number of points n_pts,
we generate it by calling rule(n_pts).
n_pts must be greater than 0.
"""
function rule(n_pts)

  fname = fname_rule(n_pts)
  fname_w = "$(fname)_w.txt"
  fname_r = "$(fname)_r.txt"
  fname_x = "$(fname)_x.txt"

  rule_exists = isfile(fname_w) && isfile(fname_r) && isfile(fname_x)

  if ! rule_exists 
    generate_nodes_weights(n_pts)
  end

  x = zeros(n_pts)
  w = zeros(n_pts)

  file = open(fname_x, "r")
  idx = 1
  while !eof(file)
    l = readline(file)
    l = strip(l)
    x[idx] = parse(Float64, l)
    idx += 1
  end
  close(file)

  file = open(fname_w, "r")
  idx = 1
  while !eof(file)
    l = readline(file)
    l = strip(l)
    w[idx] = parse(Float64, l)
    idx += 1
  end
  close(file)

  return x, w
end # function


end # GB
