function make_BoundaryConditions(rhs::RHS, prms)
  type = query(String, prms, "type")

  xrange = extrema(rhs.mesh.x)

  lhs_bc, rhs_bc = @match type begin
    "periodic"          => (PeriodicBC(), PeriodicBC())
    "from initial data" => begin
      @unpack D, S, tau = get_dynamic_variables(rhs.cache)
      @unpack rho, v, eps, p = get_static_variables(rhs.cache)
      lhs_state = (D[1], S[1], tau[1], p[1], rho[1], v[1], eps[1])
      rhs_state = (D[end], S[end], tau[end], p[end], rho[end], v[end], eps[end])
      (DirichletBC(lhs_state), DirichletBC(rhs_state))
    end
    _ => error("Boundary condition type '$type' not implemented!")
  end

  return BoundaryConditions(lhs_bc, rhs_bc, rhs.rsolver)
end


function make_BoundaryConditions_LDG(ldg_rsolver, prms)
  isnothing(ldg_rsolver) && return nothing
  type = query(String, prms, "type")
  lhs_bc, rhs_bc = if type == "periodic"
    PeriodicBC(), PeriodicBC()
  else
    OutflowBC(), OutflowBC()
  end
  return BoundaryConditions(lhs_bc, rhs_bc, ldg_rsolver)
end
