@testset "EquationOfState" begin


@testset "Variables" begin

  # Call all methods we expect an eqaution of state to implement at least once here.
  for EOS in concretetypes(AbstractEquationOfState)

    parameters = parse_parameters(EOS)
    eos = EOS(parameters...)

    rho    = 1.0
    eps    = 1.0
    p      = eos(Pressure, Density, rho, InternalEnergy, eps)
    h      = eos(Enthalpy, Density, rho, InternalEnergy, eps)
    if !(EOS <: HybridEoS)
      eps = eos(InternalEnergy, Density, rho, Pressure, p)
    end
    dpdrho = eos(Derivative{Pressure, Density}, Density, rho, InternalEnergy, eps)
    dpdrho = eos(Derivative{Pressure, InternalEnergy}, Density, rho, InternalEnergy, eps)

    # derived quantities
    h      = eos(Enthalpy, Density, rho, InternalEnergy, eps)
    cs     = eos(SpeedOfSound, Density, rho, InternalEnergy, eps)
    cs2    = eos(SquareSpeedOfSound, Density, rho, InternalEnergy, eps)
    e      = eos(TotalEnergy, Density, rho, InternalEnergy, eps)

    # call combinations we know should not be implemented
    @test_throws MethodError eos(Enthalpy, Pressure, p, InternalEnergy, eps) # wrong argument ordering
    # unknown derivative (but could in principle be implemented)
    @test_throws MethodError eos(Derivative{Enthalpy, Pressure}, Density, rho, Pressure, p)

  end

end


end
